#ifndef THERMOSTAT_H

#include <Arduino.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/io.h>
#include <avr/wdt.h>
#include <Time.h>
#include <Ethernet.h>
#include <EthernetUdp.h>
#include <Dns.h>
#include <SPI.h>
#include <SD.h>

#include <HardwareSerial.h>
#include <EEPROM.h>
#include <Timezone.h>
#include <DS1302.h>

#define MAXZONES 3

#define DS18S20DEBUG 0
#define EthDEBUG 1
#define EthHardCoded 0
#define NTPDEBUG 1
#define TimerDEBUG 0
#define GET_1302_DEBUG 1
#define SDDEBUG 0
#define UIDEBUG 0
#define KBDDEBUG 0
#define BLDEBUG 0
#define DHTDEBUG 0
#define EthRESET 1

#define SDCARD 0

#define EthResPin  A1
#define Boiler0Pin 6
#define Boiler1Pin 7
#define Boiler2Pin 9
#define SDCARD_PIN 4
// 10,11,12,13,4 used by ethernet shield and sdcard.  (UNO)
// 10,50,51,52,53,4 used by ethernet shield and sdcard.  (Mega)
#define rxSense 11
#define DHT22_0_PIN 2
#define DHT22_1_PIN 8
#define DHT22_2_PIN 5
#define ONEWIRE_PIN 3
// 22,24,26,28,30,32 used for keyboard scanning
#define DNS_OK_LED 42
#define USE_DHCP_BUTTON 43

#define BoilerInvert 1 // to correct for relay drive inversion

void HandleBoiler (unsigned char);
void HandleDisplay(unsigned char, unsigned char, unsigned char);
void vacation (unsigned char);

#define ON 1
#define OFF 0
#define CHECK 5
#define HEATING 56

#if IS_MAIN
    #define EXTERN
    #define INIT(x) =x
    #define INIT3(x,y,z) =x,y,z
#else
    #define EXTERN extern
    #define INIT(x)
    #define INIT3(x,y,z)
#endif

#define MAXMSTIMERS 3
#define KBUITIMER 0
#define TOD_TIMER 1
#define TEMPTIMER 2
EXTERN unsigned int msec_Timers[MAXMSTIMERS];
#define MAXSTIMERS MAXZONES + 1
#define ZONE_0_TIMER 0
#define ZONE_1_TIMER 1
#define ZONE_2_TIMER 2
#define MISC_TIMER 3
EXTERN unsigned int sec_Timers[MAXSTIMERS];
#define MAXMTIMERS 1
#define VACATION_TIMER 0
EXTERN unsigned long min_Timers[MAXMTIMERS];
EXTERN time_t tNow;
EXTERN int Cvalue[MAXZONES], Fvalue[MAXZONES], Hvalue[MAXZONES];
EXTERN uint8_t outlier_count[MAXZONES];
EXTERN float Ffvalue[MAXZONES];
EXTERN char Dbuf[150];  //general purpose buffer for strings
EXTERN char Dbuf2[50];	// all the rest for building messages with sprintf
EXTERN char Dbuf3[300];
EXTERN char Dbuf4[20];
EXTERN char Dbuf5[300];
EXTERN char SensorUnplugged[MAXZONES];
EXTERN unsigned char vweek, vdays, vhour, vmin;

#define MAXSETPT 4
typedef struct setpt
{
  // unsigned char day;	// maybe someday
  unsigned char hour;
  unsigned char minute;
  unsigned char temp;
  unsigned char active;
} SETPT;

enum TZindex { Eastern, Central, Mountain, Arizona, Pacific };

#define CONFIG_REVISION 12362L
typedef struct zconfig_t
{
  unsigned char v_tempSetting;	// copy of tempSetting for vacation mode use
  unsigned char tempSetting;
  int modeSetting; //heat or cool
  int Hysteresis; //kick in at one degree beyond setting and out at one degree past it
  float TempFudge;
  SETPT setpoint[MAXSETPT];
  char auto_man;	// 'a' for auto, 'm' for manual (hold mode)
  unsigned char backlight;
  unsigned char dimbacklight;
  unsigned char rows;
  unsigned char cols;
  unsigned char is_dht22;	// if temp sensor is DHT22
  unsigned char clock_is_set;	// if ds1302 clock has been set
} CONFIGBLK;
typedef struct config_t
{
  long magic_number;
  time_t vacation_end_time;
  TZindex tzindex;
  uint8_t ip[4];
  uint8_t dns_server[4];
  uint8_t gateway_ip[4];
  uint8_t subnet_mask[4];
  uint8_t mac[6];
  uint8_t mqtt_server[4];
  char mqtt_user[20];
  char mqtt_pass[20];
  uint16_t mqtt_port;
} CONFIGGEN;

EXTERN CONFIGBLK zone_config[MAXZONES];
EXTERN CONFIGGEN general_config;
EXTERN byte state_saved[MAXZONES];
EXTERN unsigned char boiler2Dpin[MAXZONES] INIT3({ Boiler0Pin, Boiler1Pin, Boiler2Pin });
EXTERN unsigned char dht22_2Dpin[MAXZONES] INIT3({ DHT22_0_PIN, DHT22_1_PIN, DHT22_2_PIN });
EXTERN unsigned char conversion_started[MAXZONES];
EXTERN unsigned char datalog_done[MAXZONES];
EXTERN unsigned char UIstate[MAXZONES];
EXTERN unsigned char prog_zone[MAXZONES];
EXTERN unsigned char prog_field[MAXZONES];
EXTERN unsigned char last_keypress[MAXZONES];
EXTERN unsigned char any_key_hit[MAXZONES];
EXTERN unsigned char has_displayed[MAXZONES];
EXTERN unsigned char keybdRowApin[MAXZONES] INIT3({ 22, 26, 30 });
EXTERN unsigned char keybdRowBpin[MAXZONES] INIT3({ 24, 28, 32 });
EXTERN unsigned int backlight_timer[MAXZONES];
EXTERN unsigned char turn_off_backlight[MAXZONES];
EXTERN unsigned char turn_on_backlight[MAXZONES];
EXTERN double Fahrenheit(double);
EXTERN double dewPointFast(double, double);
EXTERN char Status[MAXZONES][8];
EXTERN unsigned char dns_working;
EXTERN int startup_delay;

#define ACT_FLD 0
#define HOUR_FLD 1
#define MIN_FLD 2
#define TEMP_FLD 3
#define MAX_FIELDS 4
// for programming the vacation duration:
#define VWEEK_FLD 0
#define VDAYS_FLD 1
#define VHOUR_FLD 2
#define VMIN_FLD 3
EXTERN unsigned char blinking[MAXZONES][4];	// active, hour, min, temp

#define BLTIMEOUT 300	// backlight timeout (in seconds)
#define DIMBACKLIGHT 0x10

#define RUN   0
#define PROG1 1
#define PROG2 2
#define PROG3 3
#define PROG4 4
#define PROGV 5

#define DL(x) Serial.print(x)
#define DLn(x) Serial.println(x)
#define DLF(x) Serial.print(F(x))
#define DLFn(x) Serial.println(F(x))
#define DV(m, v) do{Serial.print(m);Serial.print(v);Serial.print(" ");}while(0)
#define DVn(m, v) do{Serial.print(m);Serial.println(v);}while(0)
#define DVF(m, v) do{Serial.print(F(m));Serial.print(v);Serial.print(F(" "));}while(0)
#define DVFn(m, v) do{Serial.print(F(m));Serial.println(v);}while(0)

time_t timelocal(time_t);

#define THERMOSTAT_H 1
#endif
